@@ -1,8 +1,6 @@
 ################################################################################
 #
-# File:    LogDir
-# Date:    $Date: 2008-04-25 08:22:51 -0500 (Fri, 25 Apr 2008) $
-# Version: $Revision: 128 $
+# $Id: LogDir.pm 33 2008-05-26 20:48:52Z aijaz $
 # 
 ################################################################################
 
@@ -29,48 +27,6 @@ OR
 
 man TaskForest
 
-If you're a developer and you want to understand the code, I would
-recommend that you read the pods in this order:
-
-=over 4
-
-=item *
-
-TaskForest
-
-=item *
-
-TaskForest::Job
-
-=item *
-
-TaskForest::Family
-
-=item *
-
-TaskForest::TimeDependency
-
-=item *
-
-TaskForest::LogDir
-
-=item *
-
-TaskForest::Options
-
-=item *
-
-TaskForest::StringHandleTier
-
-=item *
-
-TaskForest::StringHandle
-
-=back
-
-Finally, read the documentation in the source.  Great efforts have been
-made to keep it current and relevant.
-
 =head1 DESCRIPTION
 
 This is a simple package that provides a location for the getLogDir
@@ -87,7 +43,7 @@ use Carp;
 
 BEGIN {
     use vars qw($VERSION);
-    $VERSION     = '1.09';
+    $VERSION     = '1.10';
 }
 
 my $log_dir_cached;
@@ -113,8 +69,9 @@ my $log_dir_cached;
 
 # ------------------------------------------------------------------------------
 sub getLogDir {
-    my $log_dir_root = shift;
-    if ($log_dir_cached) {
+    my ($log_dir_root, $reload) = @_;
+    
+    if ($log_dir_cached and !$reload) {
         return $log_dir_cached;
     }
     
