@@ -1,9 +1,7 @@
 ################################################################################
 #
-# File:    Rerun
-# Date:    $Date: 2008-04-25 08:22:51 -0500 (Fri, 25 Apr 2008) $
-# Version: $Revision: 128 $
-# 
+# $Id: Rerun.pm 33 2008-05-26 20:48:52Z aijaz $
+#
 ################################################################################
 
 =head1 NAME
@@ -28,48 +26,6 @@ OR
 
 man TaskForest
 
-If you're a developer and you want to understand the code, I would
-recommend that you read the pods in this order:
-
-=over 4
-
-=item *
-
-TaskForest
-
-=item *
-
-TaskForest::Job
-
-=item *
-
-TaskForest::Family
-
-=item *
-
-TaskForest::TimeDependency
-
-=item *
-
-TaskForest::LogDir
-
-=item *
-
-TaskForest::Options
-
-=item *
-
-TaskForest::StringHandleTier
-
-=item *
-
-TaskForest::StringHandle
-
-=back
-
-Finally, read the documentation in the source.  Great efforts have been
-made to keep it current and relevant.
-
 =head1 DESCRIPTION
 
 This is a simple package that provides a location for the rerun
@@ -84,10 +40,11 @@ use strict;
 use warnings;
 use Carp;
 use File::Copy;
+use TaskForest::Family;
 
 BEGIN {
     use vars qw($VERSION);
-    $VERSION     = '1.09';
+    $VERSION     = '1.10';
 }
 
 # ------------------------------------------------------------------------------
@@ -113,9 +70,41 @@ BEGIN {
 
 # ------------------------------------------------------------------------------
 sub rerun {
+    my ($family_name, $job_name, $log_dir, $cascade, $dependents_only, $family_dir) = @_;
+
+    my $jobs;
+    
+    if ($cascade or $dependents_only) {
+        $ENV{TF_JOB_DIR}     = 'unnecessary';
+        $ENV{TF_RUN_WRAPPER} = 'unnecessary';
+        $ENV{TF_LOG_DIR}     = $log_dir;
+        $ENV{TF_FAMILY_DIR}  = $family_dir;
+
+        my $family = TaskForest::Family->new(name => $family_name);
+
+        $jobs = $family->findDependentJobs($job_name);
+
+        if ($cascade) {
+            push (@$jobs, $job_name);
+        }
+
+    }
+    else {
+        $jobs = [$job_name];
+    }
+
+    foreach my $job (@$jobs) { 
+        rerunHelp($family_name, $job, $log_dir);
+    }
+}
+
+
+sub rerunHelp {
     my ($family_name, $job_name, $log_dir) = @_;
+
     my $rc = 0;
     
+    
     print "Making job $family_name $job_name available for rerun.\n";
 
     my $rc_file      = "$log_dir/$family_name.$job_name.0";
